@@ -3,7 +3,7 @@
  *
  * Home page of code is: http://smartmontools.sourceforge.net
  *
- * Copyright (C) 2002-3 Bruce Allen <smartmontools-support@lists.sourceforge.net>
+ * Copyright (C) 2002-4 Bruce Allen <smartmontools-support@lists.sourceforge.net>
  * Copyright (C) 1999-2000 Michael Cornwell <cornwell@acm.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,7 @@
 #ifndef ATACMDS_H_
 #define ATACMDS_H_
 
-#define ATACMDS_H_CVSID "$Id: atacmds.h,v 1.55 2003/10/15 14:06:02 ballen4705 Exp $\n"
+#define ATACMDS_H_CVSID "$Id: atacmds.h,v 1.63 2004/02/04 14:21:49 ballen4705 Exp $\n"
 
 #include <sys/ioctl.h>
 #include <sys/fcntl.h>
@@ -46,7 +46,9 @@ typedef enum {
   READ_THRESHOLDS,
   READ_LOG,
   IDENTIFY,
-  PIDENTIFY
+  PIDENTIFY,
+  // returns 1 byte of data
+  CHECK_POWER_MODE
 #if DEVELOP_SELECTIVE_SELF_TEST
   // writes 512 bytes of data:
   , WRITE_LOG
@@ -54,51 +56,53 @@ typedef enum {
 } smart_command_set;
 
 // ATA Specification Command Register Values (Commands)
-#define WIN_IDENTIFY            0xec						  
-#define WIN_PIDENTIFY		0xa1
-#define WIN_SMART		0xb0
+#define ATA_IDENTIFY_DEVICE             0xec                                              
+#define ATA_IDENTIFY_PACKET_DEVICE      0xa1
+#define ATA_SMART_CMD                   0xb0
+#define ATA_CHECK_POWER_MODE            0xe5
 
 // ATA Specification Feature Register Values (SMART Subcommands).
 // Note that some are obsolete as of ATA-7.
-#define SMART_READ_VALUES	0xd0
-#define SMART_READ_THRESHOLDS	0xd1
-#define SMART_AUTOSAVE		0xd2
-#define SMART_SAVE		0xd3
-#define SMART_IMMEDIATE_OFFLINE	0xd4
-#define SMART_READ_LOG_SECTOR   0xd5
-#define SMART_WRITE_LOG_SECTOR  0xd6
-#define SMART_WRITE_THRESHOLDS  0xd7
-#define SMART_ENABLE		0xd8
-#define SMART_DISABLE		0xd9
-#define SMART_STATUS		0xda
+#define ATA_SMART_READ_VALUES           0xd0
+#define ATA_SMART_READ_THRESHOLDS       0xd1
+#define ATA_SMART_AUTOSAVE              0xd2
+#define ATA_SMART_SAVE                  0xd3
+#define ATA_SMART_IMMEDIATE_OFFLINE     0xd4
+#define ATA_SMART_READ_LOG_SECTOR       0xd5
+#define ATA_SMART_WRITE_LOG_SECTOR      0xd6
+#define ATA_SMART_WRITE_THRESHOLDS      0xd7
+#define ATA_SMART_ENABLE                0xd8
+#define ATA_SMART_DISABLE               0xd9
+#define ATA_SMART_STATUS                0xda
 // SFF 8035i Revision 2 Specification Feature Register Value (SMART
 // Subcommand)
-#define SMART_AUTO_OFFLINE	0xdb
+#define ATA_SMART_AUTO_OFFLINE          0xdb
 
-// Sector Number values for SMART_IMMEDIATE_OFFLINE Subcommand
-#define OFFLINE_FULL_SCAN		0
-#define SHORT_SELF_TEST			1
-#define EXTEND_SELF_TEST		2
+// Sector Number values for ATA_SMART_IMMEDIATE_OFFLINE Subcommand
+#define OFFLINE_FULL_SCAN               0
+#define SHORT_SELF_TEST                 1
+#define EXTEND_SELF_TEST                2
 #define CONVEYANCE_SELF_TEST            3
 #define SELECTIVE_SELF_TEST             4
 #define ABORT_SELF_TEST                 127
-#define SHORT_CAPTIVE_SELF_TEST		129
-#define EXTEND_CAPTIVE_SELF_TEST	130
+#define SHORT_CAPTIVE_SELF_TEST         129
+#define EXTEND_CAPTIVE_SELF_TEST        130
 #define CONVEYANCE_CAPTIVE_SELF_TEST    131
 #define SELECTIVE_CAPTIVE_SELF_TEST     132
 #define CAPTIVE_MASK                    (0x01<<7)
 
 // Maximum allowed number of SMART Attributes
-#define NUMBER_ATA_SMART_ATTRIBUTES 	30
+#define NUMBER_ATA_SMART_ATTRIBUTES     30
 
 // Needed parts of the ATA DRIVE IDENTIFY Structure. Those labeled
 // word* are NOT used.
+#pragma pack(1)
 struct ata_identify_device {
   unsigned short words000_009[10];
-  unsigned char	 serial_no[20];
+  unsigned char  serial_no[20];
   unsigned short words020_022[3];
-  unsigned char	 fw_rev[8];
-  unsigned char	 model[40];
+  unsigned char  fw_rev[8];
+  unsigned char  model[40];
   unsigned short words047_079[33];
   unsigned short major_rev_num;
   unsigned short minor_rev_num;
@@ -110,6 +114,7 @@ struct ata_identify_device {
   unsigned short csf_default;
   unsigned short words088_255[168];
 };
+#pragma pack()
 
 /* ata_smart_attribute is the vendor specific in SFF-8035 spec */ 
 #pragma pack(1)
@@ -204,7 +209,7 @@ struct ata_smart_threshold_entry {
 /* Format of Read SMART THreshold Command */
 /* Compare to ata_smart_values above */
 #pragma pack(1)
-struct ata_smart_thresholds {
+struct ata_smart_thresholds_pvt {
   unsigned short int revnumber;
   struct ata_smart_threshold_entry thres_entries[NUMBER_ATA_SMART_ATTRIBUTES];
   unsigned char reserved[149];
@@ -330,15 +335,18 @@ struct ata_selective_self_test_log {
 };
 #pragma pack()
 
-/* Read S.M.A.R.T information from drive */
+// Get information from drive
 int ataReadHDIdentity(int device, struct ata_identify_device *buf);
+int ataCheckPowerMode(int device);
+
+/* Read S.M.A.R.T information from drive */
 int ataReadSmartValues(int device,struct ata_smart_values *);
-int ataReadSmartThresholds(int device, struct ata_smart_thresholds *);
+int ataReadSmartThresholds(int device, struct ata_smart_thresholds_pvt *);
 int ataReadErrorLog(int device, struct ata_smart_errorlog *);
 int ataReadSelfTestLog(int device, struct ata_smart_selftestlog *);
 int ataSmartStatus(int device);
-int ataSetSmartThresholds(int device, struct ata_smart_thresholds *);
-int ataReadLogDirectory(int device, struct ata_smart_log_directory *);	
+int ataSetSmartThresholds(int device, struct ata_smart_thresholds_pvt *);
+int ataReadLogDirectory(int device, struct ata_smart_log_directory *);  
 
 /* Enable/Disable SMART on device */
 int ataEnableSmart ( int device );
@@ -377,7 +385,7 @@ int ataIsSmartEnabled(struct ata_identify_device *drive);
 /* Check SMART for Threshold failure */
 // onlyfailed=0 : are or were any age or prefailure attributes <= threshold
 // onlyfailed=1:  are any prefailure attributes <= threshold now
-int ataCheckSmart ( struct ata_smart_values *data, struct ata_smart_thresholds *thresholds, int onlyfailed);
+int ataCheckSmart ( struct ata_smart_values *data, struct ata_smart_thresholds_pvt *thresholds, int onlyfailed);
 
 int ataSmartStatus2(int device);
 
@@ -389,7 +397,9 @@ int isShortSelfTestTime ( struct ata_smart_values *data);
 
 int isExtendedSelfTestTime ( struct ata_smart_values *data);
 
-int isSmartErrorLogCapable ( struct ata_smart_values *data);
+int isSmartErrorLogCapable(struct ata_smart_values *data, struct ata_identify_device *identity);
+
+int isSmartTestLogCapable(struct ata_smart_values *data, struct ata_identify_device *identity);
 
 int isGeneralPurposeLoggingCapable(struct ata_identify_device *identity);
 
@@ -414,8 +424,8 @@ int TestTime(struct ata_smart_values *data,int testtype);
 // Prints the raw value (with appropriate formatting) into the
 // character string out.
 long long ataPrintSmartAttribRawValue(char *out, 
-				      struct ata_smart_attribute *attribute,
-				      unsigned char *defs);
+                                      struct ata_smart_attribute *attribute,
+                                      unsigned char *defs);
 
 // Prints Attribute Name for standard SMART attributes. Writes a
 // 30 byte string with attribute name into output
@@ -428,8 +438,8 @@ void ataPrintSmartAttribName(char *output, unsigned char id, unsigned char *defi
 // prefail attribute.  Else we return minus the attribute number if it
 // is a usage attribute.
 int ataCheckAttribute(struct ata_smart_values *data,
-		      struct ata_smart_thresholds *thresholds,
-		      int n);
+                      struct ata_smart_thresholds_pvt *thresholds,
+                      int n);
 
 // External handler function, for when a checksum is not correct.  Can
 // simply return if no action is desired, or can print error messages
@@ -458,5 +468,6 @@ char *create_vendor_attribute_arg_list(void);
 int ata_command_interface(int device, smart_command_set command, int select, char *data);
 int escalade_command_interface(int fd, int disknum, smart_command_set command, int select, char *data);
 
-
+// This function is exported to give low-level capability
+int smartcommandhandler(int device, smart_command_set command, int select, char *data);
 #endif /* ATACMDS_H_ */
