@@ -35,7 +35,7 @@
 #include "knowndrives.h"
 #include "config.h"
 
-const char *ataprint_c_cvsid="$Id: ataprint.c,v 1.135 2004/02/15 21:15:20 ballen4705 Exp $"
+const char *ataprint_c_cvsid="$Id: ataprint.c,v 1.140 2004/03/06 22:08:00 ballen4705 Exp $"
 ATACMDNAMES_H_CVSID ATACMDS_H_CVSID ATAPRINT_H_CVSID CONFIG_H_CVSID EXTERN_H_CVSID KNOWNDRIVES_H_CVSID SMARTCTL_H_CVSID UTILITY_H_CVSID;
 
 // for passing global control variables
@@ -126,8 +126,25 @@ char *construct_st_er_desc(struct ata_smart_errorlog_struct *data) {
   unsigned char ST=data->error_struct.status;
   unsigned char ER=data->error_struct.error_register;
   char *s;
-  char *error_flag[8];
-  int i, print_lba=0;
+  const char *error_flag[8];
+  int i, print_lba=0, print_sector=0;
+
+  // Set of character strings corresponding to different error codes.
+  // Please keep in alphabetic order if you add more.
+  const char  *abrt  = "ABRT";  // ABORTED
+ const char   *amnf  = "AMNF";  // ADDRESS MARK NOT FOUND
+ const char   *ccto  = "CCTO";  // COMMAND COMPLETTION TIMED OUT
+ const char   *eom   = "EOM";   // END OF MEDIA
+ const char   *icrc  = "ICRC";  // INTERFACE CRC ERROR
+ const char   *idnf  = "IDNF";  // ID NOT FOUND
+ const char   *ili   = "ILI";   // MEANING OF THIS BIT IS COMMAND-SET SPECIFIC
+ const char   *mc    = "MC";    // MEDIA CHANGED 
+ const char   *mcr   = "MCR";   // MEDIA CHANGE REQUEST
+ const char   *nm    = "NM";    // NO MEDIA
+ const char   *obs   = "obs";   // OBSOLETE
+ const char   *tk0nf = "TK0NF"; // TRACK 0 NOT FOUND
+ const char   *unc   = "UNC";   // UNCORRECTABLE
+ const char   *wp    = "WP";    // WRITE PROTECTED
 
   /* If for any command the Device Fault flag of the status register is
    * not used then used_device_fault should be set to 0 (in the CR switch
@@ -140,37 +157,113 @@ char *construct_st_er_desc(struct ata_smart_errorlog_struct *data) {
     error_flag[i] = NULL;
 
   switch (CR) {
+  case 0x10:  // RECALIBRATE
+    error_flag[2] = abrt;
+    error_flag[1] = tk0nf;
+    break;
   case 0x20:  /* READ SECTOR(S) */
+  case 0x21:  // READ SECTOR(S)
+  case 0x24:  // READ SECTOR(S) EXT
   case 0xC4:  /* READ MULTIPLE */
-    error_flag[6] = "UNC";
-    error_flag[5] = "MC";
-    error_flag[4] = "IDNF";
-    error_flag[3] = "MCR";
-    error_flag[2] = "ABRT";
-    error_flag[1] = "NM";
-    error_flag[0] = "obs";
+  case 0x29:  // READ MULTIPLE EXT
+    error_flag[6] = unc;
+    error_flag[5] = mc;
+    error_flag[4] = idnf;
+    error_flag[3] = mcr;
+    error_flag[2] = abrt;
+    error_flag[1] = nm;
+    error_flag[0] = amnf;
     print_lba=1;
     break;
+  case 0x22:  // READ LONG (with retries)
+  case 0x23:  // READ LONG (without retries)
+    error_flag[4] = idnf;
+    error_flag[2] = abrt;
+    error_flag[0] = amnf;
+    print_lba=1;
+    break;
+  case 0x2a:  // READ STREAM DMA
+  case 0x2b:  // READ STREAM PIO
+    if (CR==0x2a)
+      error_flag[7] = icrc;
+    error_flag[6] = unc;
+    error_flag[5] = mc;
+    error_flag[4] = idnf;
+    error_flag[3] = mcr;
+    error_flag[2] = abrt;
+    error_flag[1] = nm;
+    error_flag[0] = ccto;
+    print_lba=1;
+    print_sector=(int)data->error_struct.sector_count;
+    break;
+  case 0x3A:  // WRITE STREAM DMA
+  case 0x3B:  // WRITE STREAM PIO
+    if (CR==0x3A)
+      error_flag[7] = icrc;
+    error_flag[6] = wp;
+    error_flag[5] = mc;
+    error_flag[4] = idnf;
+    error_flag[3] = mcr;
+    error_flag[2] = abrt;
+    error_flag[1] = nm;
+    error_flag[0] = ccto;
+    print_lba=1;
+    print_sector=(int)data->error_struct.sector_count;
+    break;
   case 0x25:  /* READ DMA EXT */
+  case 0x26:  // READ DMA QUEUED EXT
+  case 0xC7:  // READ DMA QUEUED
   case 0xC8:  /* READ DMA */
-    error_flag[7] = "ICRC";
-    error_flag[6] = "UNC";
-    error_flag[5] = "MC";
-    error_flag[4] = "IDNF";
-    error_flag[3] = "MCR";
-    error_flag[2] = "ABRT";
-    error_flag[1] = "NM";
-    error_flag[0] = "obs";
+  case 0xC9:
+    error_flag[7] = icrc;
+    error_flag[6] = unc;
+    error_flag[5] = mc;
+    error_flag[4] = idnf;
+    error_flag[3] = mcr;
+    error_flag[2] = abrt;
+    error_flag[1] = nm;
+    error_flag[0] = amnf;
     print_lba=1;
+    if (CR==0x25 || CR==0xC8)
+      print_sector=(int)data->error_struct.sector_count;
     break;
   case 0x30:  /* WRITE SECTOR(S) */
+  case 0x31:  // WRITE SECTOR(S)
+  case 0x34:  // WRITE SECTOR(S) EXT
   case 0xC5:  /* WRITE MULTIPLE */
-    error_flag[6] = "WP";
-    error_flag[5] = "MC";
-    error_flag[4] = "IDNF";
-    error_flag[3] = "MCR";
-    error_flag[2] = "ABRT";
-    error_flag[1] = "NM";
+  case 0x39:  // WRITE MULTIPLE EXT
+  case 0xCE:  // WRITE MULTIPLE FUA EXT
+    error_flag[6] = wp;
+    error_flag[5] = mc;
+    error_flag[4] = idnf;
+    error_flag[3] = mcr;
+    error_flag[2] = abrt;
+    error_flag[1] = nm;
+    print_lba=1;
+    break;
+  case 0x32:  // WRITE LONG (with retries)
+  case 0x33:  // WRITE LONG (without retries)
+    error_flag[4] = idnf;
+    error_flag[2] = abrt;
+    print_lba=1;
+    break;
+  case 0x3C:  // WRITE VERIFY
+    error_flag[6] = unc;
+    error_flag[4] = idnf;
+    error_flag[2] = abrt;
+    error_flag[0] = amnf;
+    print_lba=1;
+    break;
+  case 0x40: // READ VERIFY SECTOR(S) with retries
+  case 0x41: // READ VERIFY SECTOR(S) without retries
+  case 0x42: // READ VERIFY SECTOR(S) EXT
+    error_flag[6] = unc;
+    error_flag[5] = mc;
+    error_flag[4] = idnf;
+    error_flag[3] = mcr;
+    error_flag[2] = abrt;
+    error_flag[1] = nm;
+    error_flag[0] = amnf;
     print_lba=1;
     break;
   case 0xA0:  /* PACKET */
@@ -183,32 +276,53 @@ char *construct_st_er_desc(struct ata_smart_errorlog_struct *data) {
     error_flag[6] = "Sense key (bit 2)",
     error_flag[5] = "Sense key (bit 1)",
     error_flag[4] = "Sense key (bit 0)",
-    error_flag[2] = "ABRT";
-    error_flag[1] = "EOM";
-    error_flag[0] = "ILI";
+    error_flag[2] = abrt;
+    error_flag[1] = eom;
+    error_flag[0] = ili;
     break;
   case 0xA1:  /* IDENTIFY PACKET DEVICE */
   case 0xEF:  /* SET FEATURES */
   case 0x00:  /* NOP */
   case 0xC6:  /* SET MULTIPLE MODE */
-    error_flag[2] = "ABRT";
+    error_flag[2] = abrt;
+    break;
+  case 0x2F:  // READ LOG EXT
+    error_flag[6] = unc;
+    error_flag[4] = idnf;
+    error_flag[2] = abrt;
+    error_flag[0] = obs;
+    break;
+  case 0x3F:  // WRITE LOG EXT
+    error_flag[4] = idnf;
+    error_flag[2] = abrt;
+    error_flag[0] = obs;
     break;
   case 0xB0:  /* SMART */
     switch(FR) {
+    case 0xD0:  // SMART READ DATA
+    case 0xD1:  // SMART READ ATTRIBUTE THRESHOLDS
     case 0xD5:  /* SMART READ LOG */
-      error_flag[6] = "UNC";
-      error_flag[4] = "IDNF";
-      error_flag[2] = "ABRT";
-      error_flag[0] = "obs";
+      error_flag[6] = unc;
+      error_flag[4] = idnf;
+      error_flag[2] = abrt;
+      error_flag[0] = obs;
       break;
     case 0xD6:  /* SMART WRITE LOG */
-      error_flag[4] = "IDNF";
-      error_flag[2] = "ABRT";
-      error_flag[0] = "obs";
+      error_flag[4] = idnf;
+      error_flag[2] = abrt;
+      error_flag[0] = obs;
       break;
+    case 0xD2:  // Enable/Disable Attribute Autosave
+    case 0xD3:  // SMART SAVE ATTRIBUTE VALUES (ATA-3)
+    case 0xD8:  // SMART ENABLE OPERATIONS
     case 0xD9:  /* SMART DISABLE OPERATIONS */
     case 0xDA:  /* SMART RETURN STATUS */
-      error_flag[2] = "ABRT";
+    case 0xDB:  // Enable/Disable Auto Offline (SFF)
+      error_flag[2] = abrt;
+      break;
+    case 0xD4:  // SMART EXECUTE IMMEDIATE OFFLINE
+      error_flag[4] = idnf;
+      error_flag[2] = abrt;
       break;
     default:
       return NULL;
@@ -218,7 +332,7 @@ char *construct_st_er_desc(struct ata_smart_errorlog_struct *data) {
   case 0xB1:  /* DEVICE CONFIGURATION */
     switch (FR) {
     case 0xC0:  /* DEVICE CONFIGURATION RESTORE */
-      error_flag[2] = "ABRT";
+      error_flag[2] = abrt;
       break;
     default:
       return NULL;
@@ -226,15 +340,27 @@ char *construct_st_er_desc(struct ata_smart_errorlog_struct *data) {
     }
     break;
   case 0xCA:  /* WRITE DMA */
-    error_flag[7] = "ICRC";
-    error_flag[6] = "WP";
-    error_flag[5] = "MC";
-    error_flag[4] = "IDNF";
-    error_flag[3] = "MCR";
-    error_flag[2] = "ABRT";
-    error_flag[1] = "NM";
-    error_flag[0] = "obs";
+  case 0xCB:
+  case 0x35:  // WRITE DMA EXT
+  case 0x3D:  // WRITE DMA FUA EXT
+  case 0xCC:  // WRITE DMA QUEUED
+  case 0x36:  // WRITE DMA QUEUED EXT
+  case 0x3E:  // WRITE DMA QUEUED FUA EXT
+    error_flag[7] = icrc;
+    error_flag[6] = wp;
+    error_flag[5] = mc;
+    error_flag[4] = idnf;
+    error_flag[3] = mcr;
+    error_flag[2] = abrt;
+    error_flag[1] = nm;
+    error_flag[0] = amnf;
     print_lba=1;
+    if (CR==0x35)
+      print_sector=(int)data->error_struct.sector_count;
+    break;
+  case 0xE4: // READ BUFFER
+  case 0xE8: // WRITE BUFFER
+    error_flag[2] = abrt;
     break;
   default:
     return NULL;
@@ -283,6 +409,12 @@ char *construct_st_er_desc(struct ata_smart_errorlog_struct *data) {
     // bits 0-7:   SN
     lba  |= data->error_struct.sector_number;
 
+    // print number of sectors, if known, and append to print string
+    if (print_sector) {
+      snprintf(tmp, 128, " %d sectors", print_sector);
+      strcat(s, tmp);
+    }
+
     // print LBA, and append to print string
     snprintf(tmp, 128, " at LBA = 0x%08x = %d", lba, lba);
     strcat(s, tmp);
