@@ -3,7 +3,7 @@
  *
  * Home page of code is: http://smartmontools.sourceforge.net
  *
- * Copyright (C) 2002-3 Bruce Allen <smartmontools-support@lists.sourceforge.net>
+ * Copyright (C) 2002-4 Bruce Allen <smartmontools-support@lists.sourceforge.net>
  * Copyright (C) 1999-2000 Michael Cornwell <cornwell@acm.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,24 +25,17 @@
 #ifndef ATAPRINT_H_
 #define ATAPRINT_H_
 
-#define ATAPRINT_H_CVSID "$Id: ataprint.h,v 1.22 2003/10/12 09:10:03 ballen4705 Exp $\n"
+#define ATAPRINT_H_CVSID "$Id: ataprint.h,v 1.26 2004/01/27 15:29:16 ballen4705 Exp $\n"
 
 #include <stdio.h>
 #include <stdlib.h>
 
-// MACROS to control printing behavior
-#define QUIETON(control)  {if (control->quietmode) control->veryquietmode=0;}
-#define QUIETOFF(control) {if (control->quietmode && !control->veryquietmode) control->veryquietmode=1;}
-
-
-
-
 /* Prints ATA Drive Information and S.M.A.R.T. Capability */
 void ataPrintDriveInfo(struct ata_identify_device *);
 
 void ataPrintGeneralSmartValues(struct ata_smart_values *, struct ata_identify_device *);
 
-void ataPrintSmartThresholds(struct ata_smart_thresholds *);
+void ataPrintSmartThresholds(struct ata_smart_thresholds_pvt *);
 
 // returns number of errors in Errorlog
 int  ataPrintSmartErrorlog(struct ata_smart_errorlog *);
@@ -52,13 +45,13 @@ int ataPrintLogDirectory(struct ata_smart_log_directory *);
 void PrintSmartAttributes(struct ata_smart_values *);
 
 void PrintSmartAttribWithThres(struct ata_smart_values *,
-                                struct ata_smart_thresholds *,
-				int onlyfailed);
+                                struct ata_smart_thresholds_pvt *,
+                                int onlyfailed);
 
 // returns number of entries that had logged errors
 int ataPrintSmartSelfTestlog(struct ata_smart_selftestlog *, int allentries);
 
-void ataPseudoCheckSmart(struct ata_smart_values *, struct ata_smart_thresholds *);
+void ataPseudoCheckSmart(struct ata_smart_values *, struct ata_smart_thresholds_pvt *);
 
 // Convenience function for formatting strings from ata_identify_device.
 void formatdriveidstring(char *out, const char *in, int n);
