@@ -27,7 +27,7 @@
 #include "utility.h"
 #include "config.h"
 
-const char *knowndrives_c_cvsid="$Id: knowndrives.c,v 1.87 2004/02/16 18:56:59 pjwilliams Exp $"
+const char *knowndrives_c_cvsid="$Id: knowndrives.c,v 1.96 2004/03/06 19:43:17 ballen4705 Exp $"
                                 ATACMDS_H_CVSID ATAPRINT_H_CVSID CONFIG_H_CVSID EXTERN_H_CVSID KNOWNDRIVES_H_CVSID UTILITY_H_CVSID;
 
 #define MODEL_STRING_LENGTH                         40
@@ -113,17 +113,19 @@ const unsigned char vendoropts_Hitachi_DK23XX[][2] = {
 const char same_as_minus_F[]="Fixes byte order in some SMART data (same as -F samsung)";
 const char same_as_minus_F2[]="Fixes byte order in some SMART data (same as -F samsung2)";
 
-const char may_need_minus_F_disabled[]="Contact developers at " PACKAGE_BUGREPORT "; may need -F samsung disabled";
-const char may_need_minus_F2_disabled[]="Contact developers at " PACKAGE_BUGREPORT "; may need -F samsung2 disabled";
+const char may_need_minus_F_disabled[]="May need -F samsung disabled; see manual for details.";
+const char may_need_minus_F2_disabled[]="May need -F samsung2 disabled; see manual for details.";
 
 /* Special-purpose functions for use in knowndrives[]. */
 void specialpurpose_reverse_samsung(smartmonctrl *con)
 {
-  con->fixfirmwarebug = FIX_SAMSUNG;
+  if (con->fixfirmwarebug==FIX_NOTSPECIFIED)
+    con->fixfirmwarebug = FIX_SAMSUNG;
 }
 void specialpurpose_reverse_samsung2(smartmonctrl *con)
 {
-  con->fixfirmwarebug = FIX_SAMSUNG2;
+  if (con->fixfirmwarebug==FIX_NOTSPECIFIED)
+    con->fixfirmwarebug = FIX_SAMSUNG2;
 }
 
 /* Table of settings for known drives terminated by an element containing all
@@ -170,6 +172,13 @@ const drivesettings knowndrives[] = {
     vendoropts_9_seconds,
     NULL, NULL
   },
+  { // Fujitsu MPC series
+    "^FUJITSU MPC3(032AT|043AT|045AH|064A[HT]|084AT|096AT|102AT)$",
+    ".*",
+    NULL,
+    vendoropts_9_seconds,
+    NULL, NULL
+  },
   { // Fujitsu MHN2300AT
     "^FUJITSU MHN2300AT$",
     ".*",
@@ -289,7 +298,7 @@ const drivesettings knowndrives[] = {
   { // All Samsung drives with '.*-25' firmware
     "^SAMSUNG.*",
     ".*-25$",
-    "May need -F samsung2 enabled; see manual page for description of -F.\n",
+    "May need -F samsung2 enabled; see manual for details.\n",
     vendoropts_9_halfminutes,
     NULL, NULL
   },
@@ -304,7 +313,7 @@ const drivesettings knowndrives[] = {
   { // Samsung ALL OTHER DRIVES
     "^SAMSUNG.*",
     ".*",
-    "Contact developers at " PACKAGE_BUGREPORT "; may need -F samsung[2] enabled.\n",
+    "May need -F samsung or -F samsung2 enabled; see manual for details.\n",
     NULL, NULL, NULL
   },
   { // Maxtor DiamondMax Plus D740X family
@@ -359,8 +368,8 @@ const drivesettings knowndrives[] = {
     vendoropts_9_minutes,
     NULL, NULL
   },
-  { // Maxtor DiamondMax Plus 40 series
-    "^Maxtor (54098H8|53073H6|52732H6|52049H4|51536H3|51369H3|51024H2)$",
+  { // Maxtor DiamondMax Plus 40 series (Ultra ATA 66 and Ultra ATA 100)
+    "^Maxtor (54098[UH]8|53073[UH]6|52732[UH]6|52049[UH]4|51536[UH]3|51369[UH]3|51024[UH]2)$",
     ".*",
     NULL,
     vendoropts_9_minutes,
@@ -395,7 +404,21 @@ const drivesettings knowndrives[] = {
     NULL, NULL
   },
   { // Maxtor DiamondMax Plus 9 family
-    "^Maxtor 6Y((060|080|120|160)L0|(080|120|160|200)P0|(060|080|120|160|200)M0)$",
+    "^Maxtor 6Y((060|080|120|160)L0|(060|080|120|160|200|250)P0|(060|080|120|160|200|250)M0)$",
+    ".*",
+    NULL,
+    vendoropts_9_minutes,
+    NULL, NULL
+  },
+  { // Maxtor MaXLine Plus II
+    "^Maxtor 7Y250[PM]0$",
+    ".*",
+    NULL,
+    vendoropts_9_minutes,
+    NULL, NULL
+  },
+  { // Maxtor MaXLine II family
+    "^Maxtor [45]A(25|32)0[JN]0$",
     ".*",
     NULL,
     vendoropts_9_minutes,
@@ -408,6 +431,11 @@ const drivesettings knowndrives[] = {
     vendoropts_Hitachi_DK23XX,
     NULL, NULL
   },
+  { // IBM Deskstar 14GXP and 16GP series
+    "^IBM-DTTA-3(7101|7129|7144|5032|5043|5064|5084|5101|5129|5168)0$",
+    ".*",
+    NULL, NULL, NULL, NULL 
+  },
   { // IBM Travelstar 25GS, 18GT, and 12GN family
     "^IBM-DARA-2(25|18|15|12|09|06)000$",
     ".*",
@@ -438,6 +466,11 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
+  { // Hitachi Deskstar 7K250 series
+    "^HDS7225((40|80|12|16)VLAT20|(12|16|25)VLAT80|(80|12|16|25)VLSA80)$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
   { // TOSHIBA MK4025GAS
     "^TOSHIBA MK4025GAS$",
     ".*",
@@ -463,8 +496,13 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
-  { // Seagate Barracuda U Series 20410
-    "^ST320410A$",
+  { // TOSHIBA MK2017GAP
+    "^TOSHIBA MK2017GAP$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // Seagate Barracuda U Series family
+    "^ST3(8002|6002|4081|2041)0A$",
     ".*",
     NULL, NULL, NULL, NULL
   },
@@ -478,6 +516,11 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
+  { // Seagate Barracuda ATA III family
+    "^ST3(40824|30620|20414|15310|10215)A$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
   { // Seagate Barracuda ATA IV family
     "^ST3(20011|40016|60021|80021)A$",
     ".*",
@@ -546,6 +589,11 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
+  { // Western Digital Caviar SE (Serial ATA) family
+    "^WDC WD(4|8|12|16|20|25)00JD-.*$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
   { // Western Digital Caviar AC38400
     "^WDC AC38400L$",
     ".*",
@@ -556,13 +604,13 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
-  { // QUANTUM FIREBALLlct15 20
-    "^QUANTUM FIREBALLlct15 20$",
+  { // QUANTUM FIREBALLlct15 20 and QUANTUM FIREBALLlct15 30
+    "^QUANTUM FIREBALLlct15 [23]0$",
     ".*",
     NULL, NULL, NULL, NULL
   },
-  { // QUANTUM FIREBALLlct20 20
-    "^QUANTUM FIREBALLlct20 20$",
+  { // QUANTUM FIREBALLlct20 20 and QUANTUM FIREBALLlct20 40
+    "^QUANTUM FIREBALLlct20 [24]0$",
     ".*",
     NULL, NULL, NULL, NULL
   },
@@ -730,7 +778,7 @@ void showpresets(const struct ata_identify_device *drive){
 // already been set in opts will not be changed.  Returns <0 if drive
 // not recognized else index >=0 into drive database.
 int applypresets(const struct ata_identify_device *drive, unsigned char **optsptr,
-                  smartmonctrl *con) {
+		 smartmonctrl *con) {
   int i;
   unsigned char *opts;
   char model[MODEL_STRING_LENGTH+1], firmware[FIRMWARE_STRING_LENGTH+1];
@@ -745,31 +793,38 @@ int applypresets(const struct ata_identify_device *drive, unsigned char **optspt
   }
   
   opts=*optsptr;
-
+  
   // get the drive's model/firmware strings
   formatdriveidstring(model, (char *)drive->model, MODEL_STRING_LENGTH);
   formatdriveidstring(firmware, (char *)drive->fw_rev, FIRMWARE_STRING_LENGTH);
-
-  // Look up the drive in knowndrives[] and check vendoropts is non-NULL.
-  if ((i = lookupdrive(model, firmware)) >= 0 && knowndrives[i].vendoropts) {
-    const unsigned char (* presets)[2];
-
-    // For each attribute in list of attribute/val pairs...
-    presets = knowndrives[i].vendoropts;
-    while (1) {
-      const int attr = (*presets)[0];
-      const int val  = (*presets)[1];
-
-      if (!attr)  
-        break;
-
-      // ... set attribute if user hasn't already done so.
-      if (!opts[attr])
-        opts[attr] = val;
-      presets++;
+  
+  // Look up the drive in knowndrives[].
+  if ((i = lookupdrive(model, firmware)) >= 0) {
+    
+    // if vendoropts is non-NULL then Attribute interpretation presets
+    if (knowndrives[i].vendoropts) {
+      const unsigned char (* presets)[2];
+      
+      // For each attribute in list of attribute/val pairs...
+      presets = knowndrives[i].vendoropts;
+      while (1) {
+	const int attr = (*presets)[0];
+	const int val  = (*presets)[1];
+	
+	if (!attr)  
+	  break;
+	
+	// ... set attribute if user hasn't already done so.
+	if (!opts[attr])
+	  opts[attr] = val;
+	presets++;
+      }
     }
-
-    // If a function is defined for this drive then call it.
+    
+    // If a special-purpose function is defined for this drive then
+    // call it. Note that if command line arguments or Directives
+    // over-ride this choice, then the specialpurpose function that is
+    // called must deal with this.
     if (knowndrives[i].specialpurpose)
       (*knowndrives[i].specialpurpose)(con);
   }
