@@ -3,11 +3,11 @@
  *
  * Home page of code is: http://smartmontools.sourceforge.net
  *
- * Copyright (C) 2002-3 Bruce Allen <smartmontools-support@lists.sourceforge.net>
+ * Copyright (C) 2002-4 Bruce Allen <smartmontools-support@lists.sourceforge.net>
  * Copyright (C) 1999-2000 Michael Cornwell <cornwell@acm.org>
  *
  * Additional SCSI work:
- * Copyright (C) 2003 Douglas Gilbert <dougg@torque.net>
+ * Copyright (C) 2003-4 Douglas Gilbert <dougg@torque.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@
 #include "utility.h"
 #include "extern.h"
 
-const char *scsicmds_c_cvsid="$Id: scsicmds.c,v 1.65 2003/11/20 01:02:27 dpgilbert Exp $" EXTERN_H_CVSID SCSICMDS_H_CVSID;
+const char *scsicmds_c_cvsid="$Id: scsicmds.c,v 1.70 2004/01/02 16:05:25 ballen4705 Exp $" EXTERN_H_CVSID SCSICMDS_H_CVSID;
 
 /* for passing global control variables */
 extern smartmonctrl *con;
@@ -262,6 +262,9 @@ int scsiLogSense(int device, int pagenum, UINT8 *pBuf, int bufLen,
         if (0 == ((pBuf[2] << 8) + pBuf[3]))
             return SIMPLE_ERR_BAD_RESP;
         pageLen = (pBuf[2] << 8) + pBuf[3] + 4;
+        /* some SCSI HBA don't like "odd" length transfers */
+        if (pageLen % 2)
+            pageLen += 1;   
         if (pageLen > bufLen)
             pageLen = bufLen;
     }
@@ -955,7 +958,7 @@ int scsiCheckIE(int device, int hasIELogPage, int hasTempLogPage,
             pout("Log Sense failed, IE page [%s]\n", scsiErrString(err));
             return err;
         }
-	// pull out page size from response, don't forget to add 4
+        // pull out page size from response, don't forget to add 4
         pagesize = (unsigned short) ((tBuf[2] << 8) | tBuf[3]) + 4; 
         if ((pagesize < 4) || tBuf[4] || tBuf[5]) {
             pout("Log Sense failed, IE page, bad parameter code or length\n");
@@ -1614,9 +1617,9 @@ void scsiDecodeErrCounterPage(unsigned char * resp,
     num = (resp[2] << 8) | resp[3];
     ucp = &resp[0] + 4;
     while (num > 3) {
-    	pc = (ucp[0] << 8) | ucp[1];
-	pl = ucp[3] + 4;
-	switch (pc) {
+        pc = (ucp[0] << 8) | ucp[1];
+        pl = ucp[3] + 4;
+        switch (pc) {
             case 0: 
             case 1: 
             case 2: 
@@ -1627,25 +1630,25 @@ void scsiDecodeErrCounterPage(unsigned char * resp,
                 ecp->gotPC[pc] = 1;
                 ullp = &ecp->counter[pc];
                 break;
-	default: 
+        default: 
                 ecp->gotExtraPC = 1;
                 ullp = &ecp->counter[7];
                 break;
-	}
-	k = pl - 4;
-	xp = ucp + 4;
-	if (k > (int)sizeof(*ullp)) {
-	    xp += (k - sizeof(*ullp));
-	    k = sizeof(*ullp);
-	}
-	*ullp = 0;
-	for (j = 0; j < k; ++j) {
-	    if (j > 0)
-	    	*ullp <<= 8;
-	    *ullp |= xp[j];
-	}
-	num -= pl;
-	ucp += pl;
+        }
+        k = pl - 4;
+        xp = ucp + 4;
+        if (k > (int)sizeof(*ullp)) {
+            xp += (k - sizeof(*ullp));
+            k = sizeof(*ullp);
+        }
+        *ullp = 0;
+        for (j = 0; j < k; ++j) {
+            if (j > 0)
+                *ullp <<= 8;
+            *ullp |= xp[j];
+        }
+        num -= pl;
+        ucp += pl;
     }
 }
 
@@ -1661,9 +1664,9 @@ void scsiDecodeNonMediumErrPage(unsigned char *resp,
     ucp = &resp[0] + 4;
     szof = sizeof(nmep->counterPC0);
     while (num > 3) {
-    	pc = (ucp[0] << 8) | ucp[1];
-	pl = ucp[3] + 4;
-	switch (pc) {
+        pc = (ucp[0] << 8) | ucp[1];
+        pl = ucp[3] + 4;
+        switch (pc) {
             case 0: 
                 nmep->gotPC0 = 1;
                 k = pl - 4;
@@ -1679,12 +1682,12 @@ void scsiDecodeNonMediumErrPage(unsigned char *resp,
                     nmep->counterPC0 |= xp[j];
                 }
                 break;
-	default: 
+        default: 
                 nmep->gotExtraPC = 1;
                 break;
-	}
-	num -= pl;
-	ucp += pl;
+        }
+        num -= pl;
+        ucp += pl;
     }
 }
 
@@ -1743,6 +1746,31 @@ int scsiCountFailedSelfTests(int fd, int noisy)
     return (fail_hour << 8) + fails;
 }
 
+/* Returns 0 if able to read self test log page; then outputs 1 into
+   *inProgress if self test still in progress, else outputs 0. */
+int scsiSelfTestInProgress(int fd, int * inProgress)
+{
+    int num;
+    UINT8 * ucp;
+    unsigned char resp[LOG_RESP_SELF_TEST_LEN];
+
+    if (scsiLogSense(fd, SELFTEST_RESULTS_LPAGE, resp, 
+                     LOG_RESP_SELF_TEST_LEN, 0))
+        return -1;
+    if (resp[0] != SELFTEST_RESULTS_LPAGE)
+        return -1;
+    // compute page length
+    num = (resp[2] << 8) + resp[3];
+    // Log sense page length 0x190 bytes
+    if (num != 0x190) {
+        return -1;
+    }
+    ucp = resp + 4;
+    if (inProgress)
+        *inProgress = (0xf == (ucp[4] & 0xf)) ? 1 : 0;
+    return 0;
+}
+
 /* Returns a negative value if failed to fetch Contol mode page or it was
    malformed. Returns 0 if GLTSD bit is zero and returns 1 if the GLTSD
    bit is set. Examines default mode page when current==0 else examines
