@@ -3,11 +3,11 @@
  *
  * Home page of code is: http://smartmontools.sourceforge.net
  *
- * Copyright (C) 2002-3 Bruce Allen <smartmontools-support@lists.sourceforge.net>
+ * Copyright (C) 2002-4 Bruce Allen <smartmontools-support@lists.sourceforge.net>
  * Copyright (C) 2000 Michael Cornwell <cornwell@acm.org>
  *
  * Additional SCSI work:
- * Copyright (C) 2003 Douglas Gilbert <dougg@torque.net>
+ * Copyright (C) 2003-4 Douglas Gilbert <dougg@torque.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
 #ifndef SCSICMDS_H_
 #define SCSICMDS_H_
 
-#define SCSICMDS_H_CVSID "$Id: scsicmds.h,v 1.43 2003/11/20 00:34:09 dpgilbert Exp $\n"
+#define SCSICMDS_H_CVSID "$Id: scsicmds.h,v 1.46 2004/01/02 16:05:25 ballen4705 Exp $\n"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -41,8 +41,6 @@
 #include <errno.h>
 #include <sys/ioctl.h>
 
-#define PROJECTHOME "http://smartmontools.sourceforge.net/"
-
 /* #define SCSI_DEBUG 1 */ /* Comment out to disable command debugging */
 
 /* Following conditional defines bypass inclusion of scsi/scsi.h and
@@ -296,6 +294,7 @@ void scsiDecodeNonMediumErrPage(unsigned char * resp,
 int scsiFetchExtendedSelfTestTime(int device, int * durationSec, 
                                   int modese_len);
 int scsiCountFailedSelfTests(int fd, int noisy);
+int scsiSelfTestInProgress(int fd, int * inProgress);
 int scsiFetchControlGLTSD(int device, int modese_len, int current);
 int scsiSetControlGLTSD(int device, int enabled, int modese_len);
 int scsiFetchTransportProtocol(int device, int modese_len);
