@@ -3,7 +3,7 @@
  *
  * Home page of code is: http://smartmontools.sourceforge.net
  *
- * Copyright (C) 2002-3 Bruce Allen <smartmontools-support@lists.sourceforge.net>
+ * Copyright (C) 2002-4 Bruce Allen <smartmontools-support@lists.sourceforge.net>
  * Copyright (C) 2000 Michael Cornwell <cornwell@acm.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,9 +25,14 @@
 #ifndef SMARTD_H_
 #define SMARTD_H_
 
+// Needed since some structure definitions below require POSIX
+// extended regular expressions.
+#include <sys/types.h>
+#include <regex.h>
+
 
 #ifndef SMARTD_H_CVSID
-#define SMARTD_H_CVSID "$Id: smartd.h,v 1.61 2003/11/19 19:38:19 ballen4705 Exp $\n"
+#define SMARTD_H_CVSID "$Id: smartd.h,v 1.69 2004/01/27 06:19:39 shattered Exp $\n"
 #endif
 
 // Configuration file
@@ -39,22 +44,12 @@
 // maximum line length in configuration file
 #define MAXLINELEN 128
 
-// maximum number of device entries in configuration file.
-#define MAXENTRIES 64
-
 // maximum length of a continued line in configuration file
 #define MAXCONTLINE 1023
 
 // default for how often SMART status is checked, in seconds
 #define CHECKTIME 1800
 
-// maximum number of ATA devices to monitor.  Under linux this should
-// not exceed 20 (/dev/hda-t).  Check against make_device_names().
-#define MAXATADEVICES	20
-
-// maximum number of SCSI devices to monitor
-#define MAXSCSIDEVICES	26
-
 /* Boolean Values */
 #define TRUE 0x01
 #define FALSE 0x00
@@ -67,17 +62,43 @@
 #define MONITOR_RAWPRINT  2
 #define MONITOR_RAW       3
 
-// If user has requested email warning messages, then this structure
-// stores the information about them.
-typedef struct mailinfo {
-  // number of times an email has been sent
-  int logged;
-  // time last email was sent, as defined by man 2 time
-  time_t lastsent;
-  // time problem initially logged
-  time_t firstsent;
+
+// Number of allowed mail message types
+#define SMARTD_NMAIL 10
+
+typedef struct mailinfo_s {
+  int logged;// number of times an email has been sent
+  time_t firstsent;// time first email was sent, as defined by time(2)
+  time_t lastsent; // time last email was sent, as defined by time(2)
 } mailinfo;
 
+// If user has requested email warning messages, then this structure
+// stores the information about them, and track type/date of email
+// messages.
+typedef struct maildata_s {
+  mailinfo maillog[10];                   // log info on when mail sent
+  char *emailcmdline;                     // script to execute
+  char *address;                          // email address, or null
+  unsigned char emailfreq;                // Emails once (1) daily (2) diminishing (3)
+  unsigned char emailtest;                // Send test email?
+} maildata;
+
+// If user has requested automatic testing, then this structure stores
+// their regular expression pattern, the compiled form of that regex,
+// and information about the disk capabilities and when the last text
+// took place
+
+typedef struct testinfo_s {
+  char *regex;                    // text form of regex
+  regex_t cregex;                 // compiled form of regex
+  unsigned short hour;            // 1+hour of year when last scheduled self-test done
+  char testtype;                  // type of test done at hour indicated just above
+  signed char not_cap_offline;    // 0==unknown OR capable of offline, 1==not capable 
+  signed char not_cap_conveyance;
+  signed char not_cap_short;
+  signed char not_cap_long;
+} testinfo;
+
 
 // cfgfile is the main data structure of smartd. It is used in two
 // ways.  First, to store a list of devices/options given in the
@@ -141,7 +162,6 @@ typedef struct mailinfo {
 // the capabilities that devices already are known to have (as noted
 // within *cfg).
 
-
 typedef struct configfile_s {
   // FIRST SET OF ENTRIES CORRESPOND TO WHAT THE USER PUT IN THE
   // CONFIG FILE.  SOME ENTRIES MAY BE MODIFIED WHEN A DEVICE IS
@@ -160,21 +180,19 @@ typedef struct configfile_s {
   char permissive;                        // Ignore failed SMART commands
   char autosave;                          // 1=disable, 2=enable Autosave Attributes
   char autoofflinetest;                   // 1=disable, 2=enable Auto Offline Test
-  unsigned char emailfreq;                // Emails once (1) daily (2) diminishing (3)
-  unsigned char emailtest;                // Send test email?
   unsigned char fixfirmwarebug;           // Fix firmware bug
   char ignorepresets;                     // Ignore database of -v options
   char showpresets;                       // Show database entry for this device
   char removable;                         // Device may disappear (not be present)
+  char powermode;                         // skip check, if disk in idle or standby mode
   unsigned char selflogcount;             // total number of self-test errors
-  unsigned char notused1[3];              // for packing alignment
   unsigned short selfloghour;             // lifetime hours of last self-test error
-  char *emailcmdline;                     // Program for sending mail (or NULL)
-  char *address;                          // Email addresses (or NULL)
-
-  // THE NEXT SET OF ENTRIES TRACK DEVICE STATE AND ARE DYNAMIC
-  mailinfo maildata[10];                  // Tracks type/date of email messages sent
+  testinfo *testdata;                     // Pointer to data on scheduled testing
   
+  
+  // THE NEXT SET OF ENTRIES ALSO TRACK DEVICE STATE AND ARE DYNAMIC
+  maildata *mailwarn;                     // non-NULL: info about sending mail or executing script
+
   // SCSI ONLY
   unsigned char SmartPageSupported;       // has log sense IE page (0x2f)
   unsigned char TempPageSupported;        // has log sense temperature page (0xd)
@@ -204,7 +222,7 @@ typedef struct configfile_s {
   // ATA ONLY - SAVE SMART DATA. NULL POINTERS UNLESS NEEDED.  IF
   // NEEDED, ALLOCATED WHEN DEVICE REGISTERED.
   struct ata_smart_values *smartval;       // Pointer to SMART data
-  struct ata_smart_thresholds *smartthres; // Pointer to SMART thresholds
+  struct ata_smart_thresholds_pvt *smartthres; // Pointer to SMART thresholds
 
 } cfgfile;
 
